Package: FastKRR
Type: Package
Title: Kernel Ridge Regression using 'RcppArmadillo'
Version: 0.2.1
Authors@R: 
  c(person(given = "Gyeongmin",
           family = "Kim",
           role = c("aut"),
           email = "rlarudals0824@gmail.com",
           comment = "Sungshin Women's University"),
    person(given = "Seyoung",
           family = "Lee",
           role = c("aut"),
           email = "sudang0404@gmail.com",
           comment = "Sungshin Women's University"),
    person(given = "Miyoung",
           family = "Jang",
           role = c("aut"),
           email = "miyoung9072@gmail.com",
           comment = "Sungshin Women's University"),
    person(given = "Kwan-Young",
           family = "Bak",
           role = c("aut", "cre", "cph"),
           email = "kybak@sungshin.ac.kr",
           comment = c(ORCID = "0000-0002-4541-160X", "Sungshin Women's University")))
Description: Provides core computational operations in C++ via 'RcppArmadillo', enabling faster performance than pure R, improved numerical stability, and parallel execution with OpenMP where available. On systems without OpenMP support, the package automatically falls back to single-threaded execution with no user configuration required. For efficient model selection, it integrates with 'CVST' to provide sequential-testing cross-validation and additionally supports restricted maximum likelihood (REML) for continuous optimization of the regularization parameter. The package offers a unified interface for exact kernel ridge regression and three scalable approximations—Nyström, Pivoted Cholesky, and Random Fourier Features—allowing analyses with substantially larger sample sizes than are feasible with exact KRR. It also integrates with the 'tidymodels' ecosystem via the 'parsnip' model specification 'krr_reg', and the S3 method tunable.krr_reg(). To understand the theoretical background, one can refer to Wainwright (2019) <doi:10.1017/9781108627771>.
License: GPL (>= 2)
URL: https://github.com/kybak90/FastKRR, https://www.tidymodels.org
BugReports: https://github.com/kybak90/FastKRR/issues
Imports: CVST, generics, parsnip, Rcpp, rlang, tibble, ggplot2
LinkingTo: Rcpp, RcppArmadillo
Suggests: knitr, rmarkdown, dials, tidymodels, modeldata, dplyr,
        testthat (>= 3.0.0)
SystemRequirements: OpenMP (optional)
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-07-21 09:10:23 UTC; KGM
Author: Gyeongmin Kim [aut] (Sungshin Women's University),
  Seyoung Lee [aut] (Sungshin Women's University),
  Miyoung Jang [aut] (Sungshin Women's University),
  Kwan-Young Bak [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-4541-160X>, Sungshin Women's
    University)
Maintainer: Kwan-Young Bak <kybak@sungshin.ac.kr>
Repository: CRAN
Date/Publication: 2026-07-21 09:30:18 UTC
