# Generated by roxygen2: do not edit by hand

S3method(coef,krr)
S3method(error,default)
S3method(error,krr)
S3method(param,default)
S3method(param,krr)
S3method(plot,krr)
S3method(predict,krr)
S3method(print,approx_kernel)
S3method(print,kernel_matrix)
S3method(print,krr)
S3method(summary,krr)
S3method(tunable,krr_reg)
S3method(update,krr_reg)
export(approx_kernel)
export(error)
export(fastkrr)
export(krr_reg)
export(make_kernel)
export(param)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(generics,tunable)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(stats,formula)
importFrom(stats,kernel)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rcauchy)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,time)
importFrom(stats,update)
importFrom(tibble,tibble)
useDynLib(FastKRR, .registration = TRUE)
