# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

make_kernel <- function(X, X_new = NULL, kernel = "gaussian", rho = 1.0, n_threads = NULL) {
    .Call(`_FastKRR_make_kernel`, X, X_new, kernel, rho, n_threads)
}

nystrom_kernel <- function(K_mm, K_nm, m_in = NULL, n_threads = 4L) {
    .Call(`_FastKRR_nystrom_kernel`, K_mm, K_nm, m_in, n_threads)
}

nystrom <- function(K_mm, K_nm, y, m, lambda, n_threads = 4L) {
    .Call(`_FastKRR_nystrom`, K_mm, K_nm, y, m, lambda, n_threads)
}

pchol_kernel <- function(X, rho = 1.0, kernel = "gaussian", m = NULL, eps = 1e-6, verbose = TRUE) {
    .Call(`_FastKRR_pchol_kernel`, X, rho, kernel, m, eps, verbose)
}

pchol <- function(X, y, lambda, rho = 1.0, kernel = "gaussian", m = NULL, eps = 1e-6, verbose = TRUE) {
    .Call(`_FastKRR_pchol`, X, y, lambda, rho, kernel, m, eps, verbose)
}

reml_exact <- function(K, y, lambda_vec, max_iter = 50L, tol = 1e-8, n_threads = 1L) {
    .Call(`_FastKRR_reml_exact`, K, y, lambda_vec, max_iter, tol, n_threads)
}

reml_lowrank <- function(F, y, lambda_vec, max_iter = 50L, tol = 1e-8, n_threads = 1L) {
    .Call(`_FastKRR_reml_lowrank`, F, y, lambda_vec, max_iter, tol, n_threads)
}

make_Z <- function(X, W, b, n_threads = 4L) {
    .Call(`_FastKRR_make_Z`, X, W, b, n_threads)
}

rff <- function(X, y, W, b, lambda, n_threads = 4L) {
    .Call(`_FastKRR_rff`, X, y, W, b, lambda, n_threads)
}

get_num_procs <- function() {
    .Call(`_FastKRR_get_num_procs`)
}

solve_chol <- function(A, b) {
    .Call(`_FastKRR_solve_chol`, A, b)
}

SOLVE_sympd <- function(A, b) {
    .Call(`_FastKRR_SOLVE_sympd`, A, b)
}

