% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.approx_kernel}
\alias{print.approx_kernel}
\title{Print method for approximated kernel matrices}
\usage{
\method{print}{approx_kernel}(x, ...)
}
\arguments{
\item{x}{An S3 object created by \code{\link{approx_kernel}}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns the input \code{approx_kernel} object after printing the options.
}
\description{
Displays the key algorithmic choices and hyperparameter options used to construct
an approximated kernel matrix object.
}
\details{
The function summarizes critical metadata attributes stored within the
\code{approx_kernel} object, including the approximation strategy (\code{opt}),
kernel type, approximation degree (\code{m}), numerical tolerance (\code{eps}),
and the number of parallel computational threads used.
}
\examples{
# Data setting
set.seed(1)
d = 1
n = 100
rho = 1
X = matrix(runif(n*d, 0, 1), nrow = n, ncol = d)

# Example: nystrom
K_nystrom = approx_kernel(X = X, opt = "nystrom", rho = rho, n_threads = 1)

print(K_nystrom)
}
\seealso{
\code{\link{approx_kernel}}, \code{\link{print.krr}}
}
