% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.krr}
\alias{summary.krr}
\title{Summary method for fitted Kernel Ridge Regression models}
\usage{
\method{summary}{krr}(object, ...)
}
\arguments{
\item{object}{An S3 object of class \code{krr}, typically returned by
\code{\link{fastkrr}}.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns the hyperparameter list or model statistics after
  printing the summary panel to the console.
}
\description{
Computes and displays a comprehensive summary of a fitted Kernel Ridge Regression (KRR) model,
including the original function call, fitted hyperparameters (via \code{\link{param.krr}}),
and the final training mean squared error (via \code{\link{error.krr}}).
}
\examples{
# Data setting
set.seed(1)
lambda = 1e-4
d = 1
n = 50
rho = 1
X = matrix(runif(n*d, 0, 1), nrow = n, ncol = d)
y = sin(2 * pi * rowMeans(X)^3) + rnorm(n, mean = 0, sd = 0.1)

data = data.frame(X, y = y)

# Example: exact
model = fastkrr(data = data, response = "y",
                 kernel = "gaussian", opt = "exact",
                 rho = rho, lambda = lambda)

summary(model)

}
\seealso{
\code{\link{fastkrr}}, \code{\link{param.krr}}, \code{\link{error.krr}}
}
